GB = 'english'
CN = 'chinese'
DE = 'german'
JP = 'japanese'
FR = 'french'
RU = 'russian'
IT = 'italian'

SUPPORT_LANG = [GB, CN, DE, JP, FR, IT]  # no support for russian temporarily due to irregular HTML tags.
