#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['cambrinary']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp', 'beautifulsoup4']

extras_require = \
{'publish': ['flit']}

entry_points = \
{'console_scripts': ['cambrinary = cambrinary.cambrinary:main']}

setup(name='cambrinary',
      version='0.1.1',
      description='A Linux terminal online dictionary, based on `Cambridge Dictionary <https://dictionary.cambridge.org>`_',
      author='xueyuanl',
      author_email='p.xueyuanl@gmail.com',
      url='https://github.com/xueyuanl/cambrinary',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.5.3',
     )
